#!/usr/bin/ruby

#this file is part of manqod
#manqod is distributed under the CDDL licence
#the author of manqod is Dobai-Pataky Balint(dpblnt@gmail.com)

#this loads the server, i test it against stability and speed

path=File::expand_path(File.join(File.dirname(__FILE__),"..",".."))
@@etc=File::expand_path(File.join(path,"etc"))
$LOAD_PATH.unshift(File::expand_path(File.join(path,"lib")))

require('thread')
Thread.abort_on_exception = true
#require('neverblock')

require('drb')
require('singleton')
require('Common/EDebug.rb')
require('Common/ManqodDB')
include Eprint
set_edebug("main","debug")
set_edebug("time","debug")
# -h argument

		# -s argument
		if idx=ARGV.index("-s")
			server_uri=ARGV[idx+1]
		end
		# -b argument
		if idx=ARGV.index("-b")
			client_uri=ARGV[idx+1]
		end
		# -c argument
		if idx=ARGV.index("-c")
			connection=ARGV[idx+1]
		end
		# -f argument
		fakes=1
		if idx=ARGV.index("-f")
			fakes=ARGV[idx+1].to_i
		end
		einfo("server_uri: #{server_uri}")
		einfo("client_uri: #{client_uri}")
		einfo("connection: #{connection}")
if idx=ARGV.index("-h") || server_uri.nil? || client_uri.nil? || connection.nil?
	print "command line parameters:\n"
	print "\tparam\tdescription\texample\n"
	print "\t-h\tthis help\n"
	print "\t-s\tserver_uri\tdruby://127.0.0.1:5549\n"
	print "\t-b\tclient_uri\tdruby://127.0.0.1:0\n"
	print "\t-c\tconnection\tbooks\n"
	print "\t-f\tfake consumers to subscribe to each model\t1\n"
	exit
end
		begin
			client_server=DRb.start_service(client_uri)
			manqod_server=DRb::DRbObject.new(nil,server_uri)
		rescue =>e
			eerror(e.message,"server_uri: #{server_uri}\nclient_uri: #{client_uri}")
			exit
		end
		begin
			einfo("server alive?: #{manqod_server.alive?}")
		rescue
			eerror("can't connect to server","server_uri: #{server_uri}")
			exit
		end

		unless connection.nil?
			begin
				conn=manqod_server.conn(connection)
				if conn.class == Hash
					ManqodDB.instance.set_manqod_db_uri(conn['uri'])
				else
					eerror("connection not available","connection: #{connection}") 
				end
			rescue =>e
				eerror(e.message,"connection: #{connection}")
				exit
			end
		end

einfo("connected")

class Array; 
	def sum; inject( nil ) { |sum,x| sum ? sum+x : x }; end;
	def avg; sum / size; end;
	def min;min=first;inject{|min,x| min >x ? x : min};end
	def max;max=first;inject{|max,x| max<x ? x : max};end
end

class FakeDrbListModel
	include DRbUndumped
	def initialize(drbmodel)
		@drbmodel=drbmodel
		@rcv=0
	end
	attr_reader :rcv
	def subscribe
		@drbmodel.subscribe(self)
	end
	def update(notifier,ids=nil,notification_subect=nil)
		#einfo("updating: #{notifier},#{ids},#{notification_subect}")
		@rcv+=1
	end
	def destroy
		@drbmodel.unsubscribe(self)
	end
	def alive?
		true
	end
end

edebug("started")
#subscriptions
fmodels=Hash.new
ManqodDB.instance.manqod_db.moditems.each_pair{|modid,moditem|
	a=Array.new
	fakes.times{
		fk=FakeDrbListModel.new(moditem)
		a.push(fk) if fk.subscribe
	}
	fmodels[modid]=a
#	einfo("#{moditem.moditem["display"]} -> #{a.size} subscribed")
}

#send lock/changed/unlock on each row of each model and fetch those models in the same time
#the longer it takes to iterate the model, the more it'll be fetched
threads=Array.new
lt=Hash.new
ManqodDB.instance.manqod_db.moditems.each_pair{|modid,moditem|
	lt[modid]=Array.new
	threads.push(Thread.new{
		t=Thread.new{
			res=moditem.filtered_fetch2
			if res.first.class.name == "Hash"
				res.each{|row|
					if row[0].class.name == "Fixnum"
						begin
							moditem.lock_iter(row[0])
							moditem.rows_changed(row[0])
							moditem.unlock_iter(row[0])
						rescue =>e
							eerror("notify:#{moditem.moditem["display"]}:#{row[0]}:#{e.message}\n#{e.backtrace.join("\n")}")
						end
					end
				}
			end
		}
		while t.alive? do
			before=Time.now
			begin
				res=moditem.filtered_fetch2
			rescue =>e
				eerror("fetch:#{moditem.moditem["display"]}:#{e.message}\n#{e.backtrace.join("\n")}")
			end
			update_time=Time.now.to_f-before.to_f
			lt[modid].push(update_time)
			sleep 1
		end
		t.join
		}
	)
}
threads.each{|t| t.join}

#results
lt.each{|modid,l|
	rcv=0
	#unsubscriptions
	fmodels[modid].each{|fake| rcv+=fake.rcv;fake.destroy}
	einfo("#{ManqodDB.instance.manqod_db.moditem(modid).moditem["display"]} -> downloads:#{l.size} (min:#{sprintf("%.2fs",l.min)} max:#{sprintf("%.2fs",l.max)} avg:#{sprintf("%.2fs",l.avg)}) (fakes:#{fmodels[modid].size}, received:#{rcv})")
}

edebug("finished")

